//-----------------------------------------------------------------------------
// Midi Theremin
// Copyright 2020 Peter Balch
// subject to the GNU General Public License
// reads sonar distances and sends commands to a VS1053 over SPI
//-----------------------------------------------------------------------------

#include <SPI.h>

//-----------------------------------------------------------------------------
// Defines and Typedefs
//-----------------------------------------------------------------------------

enum TLeftHandControl {lhVolume,lhTremulo,lhVibrato,lhPitchBendUp,lhPitchBendDown,lhReverb,lhPolyphony,lhChordSize};

#define xDEBUG

//-----------------------------------------------------------------------------
// Global Constants
//-----------------------------------------------------------------------------

const long BAUDRATE  = 9600;  // Baud rate of UART in bps

const int  XRST  = A5;
const int  SCLK  = 13;
const int  DREQ  = A1;
const int  XCS   = A0;

const byte UStrigR = 5;
const byte USechoR = 4;
const byte UStrigL = 10;
const byte USechoL = 9;
const int maxDuration = 600;
const int minDuration = 30;
const int BinMin = 0;
const byte maxPolyphony = 7;
const int PitchWheelMax = 128*12*2; // pitch-wheel max, 128 = 1 semitone 
const char chPlay = 'z'-'a';

const int MaxScaleNotes = 10;
const PROGMEM byte ScalesArray[] = {
    7, 0,2,4,5,7,9,11, 0,0,          // C, D, E, F, G, A, B, C              major_Heptatonic
    7, 0,2,3,5,7,8,10, 0,0,          // C, D, Eb, F, F#, A, Bb, C           minor_Heptatonic
    5, 0,2,5,7,9, 0,0,0,0,           // C, D, F, G, B, C                    major_Pentatonic
    5, 0,3,5,7,10, 0,0,0,0,          // C, D#, F, G, B#, C                  minor_Pentatonic
    7, 0,2,4,5,7,8,10, 0,0,          // C, D, E, F, G, Ab, Bb, C            Aeolian_Dominant_Hindu
    7, 0,2,3,5,7,8,10, 0,0,          // C, D, Eb, F, G, Ab, Bb, C           Aeolian_Mode
    8, 0,2,3,5,6,7,8,11, 0,          // C, D, Eb, F, F#, G, Ab, B, C        Algerian
    7, 0,2,4,5,6,8,10, 0,0,          // C, D, E, F, F#, G#, Bb, C           Arabic
    6, 0,3,4,7,8,11, 0,0,0,          // C, D#, E, G, Ab, B, C               Augmented
    5, 0,1,3,7,8, 0,0,0,0,           // C, Db, Eb, G, Ab, C                 Balinese
    8, 0,2,4,5,7,8,9,11, 0,          // C, D, E, F, G, G#, A, B, C          Bebop
    8, 0,2,3,4,5,7,9,10, 0,          // C, D, Eb, E, F, G, A, Bb, C         Bebop_Minor
    6, 0,3,5,6,7,10, 0,0,0,          // C, Eb, F, F#, G, Bb, C              Blues
    7, 0,1,4,5,7,8,11, 0,0,          // C, Db, E, F, G, Ab, B, C            Byzantine
    5, 0,4,6,7,11, 0,0,0,0,          // C, E, F#, G, B, C                   Chinese
    8, 0,2,3,5,6,8,9,11, 0,          // C, D, Eb, F, Gb, Ab, A, B, C        Diminished
    8, 0,1,3,4,6,7,9,10, 0,          // C, Db, D#, E, F#, G, A, Bb, C       Dominant_Diminished
    7, 0,2,3,5,7,9,10, 0,0,          // C, D, Eb, F, G, A, Bb, C            Dorian_Mode
    5, 0,2,5,7,10, 0,0,0,0,          // C, D, F, G, Bb, C                   Egyptian
    8, 0,1,3,4,5,6,8,10, 0,          // C, C#, D#, E, F, F#, G#, A#, C      Eight_Tone_Spanish
    7, 0,2,3,5,7,8,10, 0,0,          // C, D, Eb, F, G, Ab, Bb, C           Geez
    5, 0,1,5,6,10, 0,0,0,0,          // C, Db, F, Gb, Bb, C                 Hirajoshi
    7, 0,2,3,6,7,8,11, 0,0,          // C, D, Eb, F#, G, Ab, B, C           Hungarian
    5, 0,1,5,7,10, 0,0,0,0,          // C, C#, F, G, A#, C                  In_sen
    7, 0,2,4,5,7,9,11, 0,0,          // C, D, E, F, G, A, B, C              Ionian_Mode
    7, 0,1,3,5,6,8,10, 0,0,          // C, Db, Eb, F, Gb, Ab, Bb, C         Locrian_Mode
    7, 0,2,4,6,7,9,10, 0,0,          // C, D, E, F#, G, A, Bb, C            Lydian_Dominant
    7, 0,2,4,6,7,9,11, 0,0,          // C, D, E, F#, G, A, B, C             Lydian_Mode
    7, 0,1,4,5,7,8,11, 0,0,          // C, Db, E, F, G, Ab, B, C            Maqams
    7, 0,2,4,5,7,9,10, 0,0,          // C, D, E, F, G, A, Bb, C             Mixolydian_Mod
    7, 0,1,3,5,7,8,11, 0,0,          // C, Db, Eb, F, G, Ab, B, C           Neapolitan
    8, 0,2,3,4,6,7,8,9,11,           // C, D, Eb, E, F#, G, G#, A, B, C     Nine_Tone
    8, 0,1,3,4,6,7,9,10, 0,          // C, Db, Eb, E, Gb, G, A, Bb, C       Octatonic
    7, 0,1,4,5,6,9,10, 0,0,          // C, Db, E, F, Gb, A, Bb, C           Oriental
    7, 0,1,3,5,7,8,10, 0,0,          // C, Db, Eb, F, G, Ab, Bb, C          Phrygian_Mode
    7, 0,2,3,6,7,9,10, 0,0,          // C, D, Eb, F#, G, A, A#, C           Romanian_Minor
    7, 0,1,4,5,7,8,10, 0,0,          // C, Db, E, F, G, Ab, Bb, C           Spanish_Gypsy
    7, 0,1,3,4,6,8,10, 0,0,          // C, Db, D#, E, Gb, G#, Bb, C         Super_Locrian
    6, 0,2,4,6,8,10, 0,0,0,          // C, D, E, F#, G#, Bb, C              Whole_Tone
    5, 0,2,5,7,9, 0,0,0,0};          // C, D, F, G, A, C                    Yo

const int MaxChordNotes = 5;
const PROGMEM byte ChordsArray[] = {
    1, 0,0,0,0,   // single note
    3, 0,4,7  ,0, // Major triad
    4, 0,4,7,9,   // Major sixth
    4, 0,4,7,10,  // Dominant seventh
    4, 0,4,7,11,  // Major seventh
    3, 0,4,8  ,0, // Augmented triad
    4, 0,4,8,10,  // Augmented seventh
    3, 0,3,7  ,0, // Minor triad
    4, 0,3,7,9,   // Minor sixth
    4, 0,3,7,10,  // Minor seventh
    4, 0,3,7,11,  // Minor-major seventh
    3, 0,3,6  ,0, // Diminished triad
    4, 0,3,6,9,   // Diminished seventh
    4, 0,3,6,10}; // Half-diminished seventh
    
//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------

byte CurVelocity = 127;
byte curBin = 255;
bool Continuous = false;
bool SendNoteOnOff = false;

int numOctaves = 2;
int firstNote = 60; // e.g. 7,2,true,60 means 15 notes in total starting on mid C in C-major
//byte polyphony = 51
byte polyphony = 1;

//enum TLeftHandControl LeftHandControl = lhVolume;
//enum TLeftHandControl LeftHandControl = lhReverb;
//enum TLeftHandControl LeftHandControl = lhVibrato;
//enum TLeftHandControl LeftHandControl = lhPolyphony;
enum TLeftHandControl LeftHandControl = lhChordSize;

//number of notes in scale then offsets in semitones from start of scale
byte Scale[MaxScaleNotes] = {7, 0,2,4,5,7,9,11, 0};     // C, D, E, F, G, A, B, C           major_Heptatonic

//number of notes in Chord then offsets in semitones from start of Chord
byte Chord[MaxChordNotes] = {1, 0,0,0,0};     // single note
byte ChordMax = 255;

byte ContinCentreNote = 60; 
int ContinPitchRange = PitchWheelMax;
int PitchBend = 0; // range -8191..8192 
int PitchBendMax = PitchWheelMax;

int VibratoMax = 0; // range 0..0x2000 max size of vibrato; 0x2000/24 is one semitone 
int Vibrato = 0; // range 0..0x2000 current size of vibrato; 0x2000/24 is one semitone 
int VibratoPeriod  = 200;  // > 1 

byte TremuloSize   = 0;   // 0..127
int TremuloPeriod  = 100;  // > 1

int MenuCur0 = 0;
int MenuCur1 = 0;
bool IsMenuValue = false;
char MenuCurVar;
bool IsMenuIncrementing = false;
bool IsPlaying = false;

//-----------------------------------------------------------------------------
void setup(void)
{
  Serial.begin(57600);
  Serial.println("Midi Theremin");

  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);

  pinMode(XRST, OUTPUT);
  pinMode(MISO, OUTPUT);
  pinMode(MOSI, INPUT);
  pinMode(SCLK, OUTPUT);
  pinMode(DREQ, INPUT);
  pinMode(XCS, OUTPUT);
  pinMode(UStrigR, OUTPUT);
  pinMode(USechoR, INPUT);
  pinMode(UStrigL, OUTPUT);
  pinMode(USechoL, INPUT);

  digitalWrite(XCS, HIGH);
  digitalWrite(XRST, LOW);
  delay(10);
  digitalWrite(XRST, HIGH);

  SPI.begin ();
  SPI.setBitOrder(MSBFIRST);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);

  digitalWrite(UStrigR, LOW);
  digitalWrite(UStrigL, LOW);

  PlayNote(255,false);
  
  SetBendRange(PitchWheelMax);  
  
  if (Continuous) {
    SPItransfer(0xC0);
    SPItransfer(49);
    SetLeftHandJob(lhTremulo); 
  }
  
//  SPItransfer(0xC0);    SPItransfer(54);
}

//-----------------------------------------------------------------------------
void SPItransfer(byte b)
{
  while (digitalRead(DREQ) == 0 ) ;
  
  digitalWrite(XCS, LOW);
  SPI.transfer(b);
  SPI.transfer(0x00); // to read result
  //  delay(1);
  //delayMicroseconds(1000);
  digitalWrite(XCS, HIGH);
  delayMicroseconds(100);
}

//-----------------------------------------------------------------------------
void SPItransfer3(byte a, byte b, byte c)
{
  SPItransfer(a);
  SPItransfer(constrain(b,0,127));
  SPItransfer(constrain(c,0,127));
}

//-----------------------------------------------------------------------------
byte GetSerial() {
  byte b;
  while ( Serial.available() == 0 ) ;
  b = Serial.read();
  //  Serial.println(b);
  return b;
}

//-----------------------------------------------------------------------------
long GetSonar(byte UStrig, byte USecho) {
  long duration;

  do {
    delay(10);
  
    digitalWrite(UStrig, HIGH);
    delayMicroseconds(10);
    digitalWrite(UStrig, LOW);
    //  delayMicroseconds(1000);
  
    while (digitalRead(USecho) == LOW) ;
    duration = 0;
    while ((digitalRead(USecho) == HIGH) && (duration < maxDuration))
      duration++;
  } while (duration < 10);
  
  return duration;
}

//-----------------------------------------------------------------------------
byte BinToNote(byte bin) {
  int BinMax;
  int numScale;

  numScale = Scale[0];

  BinMax = numScale * numOctaves;
  bin = BinMax - bin;

  return firstNote + Scale[bin % numScale +1] + 12 * (bin / numScale);
}

//-----------------------------------------------------------------------------
byte DurationToBin(long duration) {
  int c, binWidth;
  int BinMax;

  if (duration >= maxDuration)
    return 255;
  if (duration <= minDuration)
    return 255;

  BinMax = Scale[0] * numOctaves;
  binWidth = ((maxDuration - minDuration) * (4 - 1) / 4 / (BinMax - BinMin));
  c = ((curBin - BinMin) * (maxDuration - minDuration) + (maxDuration - minDuration) / 2) / (BinMax - BinMin + 1) + minDuration;
  if ((duration < c + binWidth) && (duration > c - binWidth))
    return curBin;

  return (BinMax - BinMin + 1) * (duration - minDuration) / (maxDuration - minDuration) + BinMin;
}

//-----------------------------------------------------------------------------
void StartChord(byte note) {
  byte i;
  for (i=1; i<=Chord[0] && i <= ChordMax; i++)
    SPItransfer3(0x90, note+Chord[i], CurVelocity); // start note          
}

//-----------------------------------------------------------------------------
void StopChord(byte note) {
  byte i;
  for (i=1;i<=Chord[0];i++)
    SPItransfer3(0x80, note+Chord[i], CurVelocity);  // stop note
}

//-----------------------------------------------------------------------------
void PlayNote(byte note, bool start) {
  // if note = 255 then stop all notes
  // if note > 200 then set polyphony to note-200
  // else start/stop note
  static byte LiveNote[maxPolyphony] = {255};
  byte i,j;

#ifdef DEBUG
Serial.print("PlayNote("); Serial.print(note); Serial.print(","); Serial.print(start); Serial.print(") ");
#endif
  if (note <= 127) {
    if (start) { // start a note
      if (LiveNote[polyphony-1] <= 127)
{
        StopChord(LiveNote[polyphony-1]); // stop note
#ifdef DEBUG
Serial.print("Stop1 "); Serial.print(LiveNote[polyphony-1]); Serial.print(" ");
#endif
}
      LiveNote[polyphony-1] = 255;

      for (j=maxPolyphony-1; j>0; j--)
        LiveNote[j] = LiveNote[j-1]; // move all up by 1 place
      LiveNote[0] = note;
      StartChord(note); // start note          
#ifdef DEBUG
Serial.print("Start "); Serial.print(note); Serial.print(" ");
#endif
    }else{ // stop a note
      StopChord(note); // stop note
#ifdef DEBUG
Serial.print("Stop2 "); Serial.print(note); Serial.print(" ");
#endif
      for (i=0; i<maxPolyphony; i++) {
        if (LiveNote[i] == note) {
          for (j=i; j<maxPolyphony-1; j++) 
            LiveNote[j] = LiveNote[j+1]; // move all above down by one
          LiveNote[maxPolyphony-1] = 255;
        }        
      }
    }    
  }else if (note == 255) { // stop all notes
    for (i=0; i<maxPolyphony; i++) 
      if (LiveNote[i] <= 127) {
        StopChord(LiveNote[i]); // stop note
#ifdef DEBUG
Serial.print("Stop3 "); Serial.print(LiveNote[i]); Serial.print(" ");
#endif
        LiveNote[i] = 255;
      }      
  }else{ // set polyphony
    polyphony = constrain(note-200,1,maxPolyphony);
    for (i=polyphony; i<maxPolyphony; i++) {
      StopChord(LiveNote[i]); // stop note
#ifdef DEBUG
Serial.print("Stop4 "); Serial.print(LiveNote[i]); Serial.print(" ");
#endif
      LiveNote[i] = 255;
    }
  }

#ifdef DEBUG
Serial.print(" >> ");
for (i=0; i<maxPolyphony; i++) {Serial.print(LiveNote[i]); Serial.print(" "); }
Serial.println("");
#endif
}

//-----------------------------------------------------------------------------
long MedianOfSeven(long a0, long a1, long a2, long a3, long a4, long a5, long a6) {
  long tmp;
  if (a4 < a0) { tmp = a4; a4 = a0; a0 = tmp; }
  if (a5 < a1) { tmp = a5; a5 = a1; a1 = tmp; }
  if (a6 < a2) { tmp = a6; a6 = a2; a2 = tmp; }
  if (a2 < a0) { tmp = a2; a2 = a0; a0 = tmp; }
  if (a3 < a1) { tmp = a3; a3 = a1; a1 = tmp; }
  if (a6 < a4) { tmp = a6; a6 = a4; a4 = tmp; }
  if (a4 < a2) { tmp = a4; a4 = a2; a2 = tmp; }
  if (a5 < a3) { tmp = a5; a5 = a3; a3 = tmp; }
  if (a1 < a0) { tmp = a1; a1 = a0; a0 = tmp; }
  if (a3 < a2) { tmp = a3; a3 = a2; a2 = tmp; }
  if (a5 < a4) { tmp = a5; a5 = a4; a4 = tmp; }
  if (a4 < a1) { tmp = a4; a4 = a1; a1 = tmp; }
  if (a6 < a3) { tmp = a6; a6 = a3; a3 = tmp; }
  if (a4 < a3) return a4; else return a3;
}

//-----------------------------------------------------------------------------
byte GetBin(long duration) {
  // filter duration by ignoring maxDuration until there hace been 10 of them
  // then do Median Of Seven
  byte bin;
  static int numMaxes = 100;
  static long prevDuration = maxDuration;
  static long d1 = maxDuration;
  static long d2 = maxDuration;
  static long d3 = maxDuration;
  static long d4 = maxDuration;
  static long d5 = maxDuration;
  static long d6 = maxDuration;

  // filter out occasional loss of echo
  if (duration >= maxDuration) {
    numMaxes++;
    if (numMaxes < 10)
      duration = prevDuration;
  } else {
    numMaxes = 0;
  }
  prevDuration = duration;

  // convert to bin number
  bin = DurationToBin(MedianOfSeven(d1, d2, d3, d4, d5, d6, duration));
  d6 = d5;
  d5 = d4;
  d4 = d3;
  d3 = d2;
  d2 = d1;
  d1 = duration;

  return bin;
}

//-----------------------------------------------------------------------------
void SetChord(byte c) {
  byte i;
  for (i=0;i<MaxChordNotes;i++)
    Chord[i] = pgm_read_byte_near(ChordsArray+c*MaxChordNotes+i);
}

//-----------------------------------------------------------------------------
void SetScale(int c) {
  byte i;
  for (i=0;i<MaxScaleNotes;i++)
    Scale[i] = pgm_read_byte_near(ScalesArray+c*MaxScaleNotes+i);
}

//-----------------------------------------------------------------------------
void SetLeftHandJob(byte c) {
  // turn off Volume and Tremulo
  SetVolume(127); 
  TremuloSize = 0; 
  // turn off Vibrato and PitchBend
  SetBendValue(0x2000);     
  Vibrato = VibratoMax;               
  PitchBend = 0;             
  // turn off SetReverb
  SetReverb(0);  
  // Set Polyphony to 1
  PlayNote(200+1,false);    

  // allow all notes in a chord
  ChordMax = 255;
                  
  LeftHandControl = c;     
  
  switch (LeftHandControl) {
    case lhVolume: SetVolume(0); break;
    case lhReverb: SPItransfer3(0xB0, 0x0C, 127); break;
    case lhChordSize: ChordMax = 1; break;    
    case lhVibrato: Vibrato = 0; break;       
  }
}

//-----------------------------------------------------------------------------
void ExecSerial(void)
{
  long duration;
  byte b, c;

  if ( Serial.available() > 0 )
  {
    b = GetSerial();
    if (b == 0xFF) {
      b = GetSerial();
      c = GetSerial();
      switch (b) {
        case 0:
          duration = GetSonar(UStrigR, USechoR);
          if (duration < maxDuration) {
            SPItransfer3(0x90,
              (Scale[0] * numOctaves - BinMin + 1)*duration / maxDuration + BinMin,
              c);
          }
          break;
        case 1:  PlayNote(200+c,false);      break;
        case 2:  numOctaves       = c;       break;
        case 3:  PitchBendMax     = constrain(((long)c*0x2000)/24,0,0x2000); break; // c semitones 
        case 4:  firstNote        = c;       break;
        case 5:  ContinCentreNote = c;       break; 
        case 6:  ContinPitchRange = constrain(((long)c*0x2000)/24,0,0x2000); break; // c semitones 
        case 8:  Continuous       = c != 0;  break; 
        case 9:  SetScale(c);                break;
        case 10: SetLeftHandJob(c);          break;
        case 11: SetChord(c);                break;
        case 12: VibratoMax       = constrain(((long)c*0x2000)/24/4,0,0x2000); // c times 1/4 semitone = 25 cents
                 if (LeftHandControl == lhVibrato) Vibrato = 0; else Vibrato = VibratoMax; 
                 break;
        case 13: VibratoPeriod    = c*10;    break; // 100 = once per sec 
        case 14: TremuloSize      = c;       break;
        case 15: TremuloPeriod    = c*10;    break; // 100 = approx once per sec 
        case 16: SendNoteOnOff    = c != 0;  break; 
        case 17: SPItransfer(0xC0); SPItransfer(c); break; 
      }
    } else {
      SPItransfer(b);
    }
  }
}

//-----------------------------------------------------------------------------
void SetBendRange(int range)
{
  SPItransfer3(0xB0, 0x65, 0x00);
  SPItransfer3(0xB0, 0x64, 0x00);
  SPItransfer3(0xB0, 0x06, range / 128);
  SPItransfer3(0xB0, 0x26, range % 128);
  SPItransfer3(0xB0, 0x65, 0x7F);
  SPItransfer3(0xB0, 0x64, 0x7F);
}

//-----------------------------------------------------------------------------
void SetBendValue(int value)
{
  value = constrain(value,0,16383); 
  SPItransfer3(0xE0, value % 128, value / 128);
}

//-----------------------------------------------------------------------------
void SetVolume(int value)
{
  SPItransfer3(0xB0, 0x07, value);
}

//-----------------------------------------------------------------------------
void SetReverb(int value)
{
  SPItransfer3(0xB0, 0x5B, value);
}    

//-----------------------------------------------------------------------------
void ApplyTremulo(void)
{
  unsigned long time;
  static int prevTremulo = 0;
  
  if ((TremuloSize > 0) || (prevTremulo > 0)) {
    time = millis() % TremuloPeriod;
    if (time > TremuloPeriod / 2)
      time = TremuloPeriod - time;
      
    SetVolume(127 - time*TremuloSize * 2 / TremuloPeriod);    
    prevTremulo = TremuloSize;
  }
}

//-----------------------------------------------------------------------------
int CalcVibratoPitchBend(void) 
{
  long time;

  time = millis() % VibratoPeriod;

  if (time > VibratoPeriod / 2)
    time = VibratoPeriod - time;
  time = time - VibratoPeriod /4;
    
  return time*Vibrato * 4 / VibratoPeriod + PitchBend;
}

//-----------------------------------------------------------------------------
void Serialwrite2(int i) 
{
  if (SendNoteOnOff) {
    Serial.write((i / 128) + 128); 
    Serial.write(i & 127); 
  }
}

//-----------------------------------------------------------------------------
void ExecContinuous(long duration) 
{
  static int numMaxes = 100;
  static bool PlayingNote = false;
  static long prevValue = 0;
  const int nSmooth = 5;
  int Value;

  if (duration >= maxDuration) {
    numMaxes++;  // filter out occasional loss of echo
    if (numMaxes < 10)
      return;
    numMaxes = 100;

    if (PlayingNote) {
      PlayNote(255,false);
      Serialwrite2(0x3FFF); 
    }  
    PlayingNote = false;
  } else {
    numMaxes = 0;

    Value = ContinPitchRange * (duration - maxDuration) / (minDuration - maxDuration) - ContinPitchRange / 2;

    if (!PlayingNote) {
      prevValue = Value;
      PitchBend = prevValue;
      SetBendValue(0x2000+CalcVibratoPitchBend()); 
      PlayNote(ContinCentreNote,true);
      PlayingNote = true;
    }else{
      prevValue = (Value + prevValue*(nSmooth-1)) / nSmooth;
      PitchBend = prevValue;
      SetBendValue(0x2000+CalcVibratoPitchBend()); 
    }      
    Serialwrite2(0x2000+PitchBend); 
  }
}

//-----------------------------------------------------------------------------
void ExecDiscrete(long duration)
{
  byte bin;
  static int numMaxes = 100;

  bin = GetBin(duration);

  if (bin == 255) {
    PlayNote(255,false); // stop all
    if (SendNoteOnOff) Serial.write(255); 
  }

  // play note if bin number changed
  if (bin != curBin) {
    curBin = bin;
    if (bin <= 127) {
      PlayNote(BinToNote(bin),true);
      if (SendNoteOnOff) Serial.write(BinToNote(bin)); 
    }
  }

  switch (LeftHandControl) {
    case lhVibrato:
    case lhPitchBendUp:
    case lhPitchBendDown:     SetBendValue(0x2000+CalcVibratoPitchBend()); break; 
    default: if (Vibrato > 0) SetBendValue(0x2000+CalcVibratoPitchBend()); 
  }                                           
}

//-----------------------------------------------------------------------------
void ExecLeftHand(long duration)
{
  static int numMaxes = 100;
  static long prevValue = 0; // in range 0..1023
  const int nSmooth = 5;
  static bool CanSeeHand = false;
  int Value;

  if (duration >= maxDuration) {
    numMaxes++;  // filter out occasional loss of echo
    if (numMaxes < 10)
      return;
    numMaxes = 100;
    Value = 0;
    CanSeeHand = false;
  } else {
    numMaxes = 0;
    Value = 1023 * (duration - maxDuration) / (minDuration - maxDuration);
    if (!CanSeeHand) 
      prevValue = Value;
    CanSeeHand = true;
  }

  if (Value != prevValue) {        
    prevValue = (Value + prevValue*(nSmooth-1)) / nSmooth;

    switch (LeftHandControl) {
      case lhVolume:        SetVolume(prevValue/8);                     break;
      case lhTremulo:       TremuloSize = prevValue/8;                  break;
      case lhVibrato:       Vibrato     = prevValue*VibratoMax/1023;    break; 
      case lhPitchBendUp:   PitchBend   = +prevValue*PitchBendMax/1023; break; 
      case lhPitchBendDown: PitchBend   = -prevValue*PitchBendMax/1023; break; 
      case lhReverb:        SetReverb(prevValue/8);                     break;
      case lhPolyphony:     PlayNote(200+prevValue/160,false);          break;    
      case lhChordSize:     ChordMax    = prevValue*Chord[0]/1023 + 1;  break;          
    }
  }
}

//-----------------------------------------------------------------------------
// loop
//-----------------------------------------------------------------------------
void loop(void)
{
  long durationR,durationL;

  ExecSerial();

  durationR = GetSonar(UStrigR, USechoR);
  durationL = GetSonar(UStrigL, USechoL);

  ExecLeftHand(durationL);

  if (TremuloSize > 0)
    ApplyTremulo();

  if (Continuous) {
    ExecContinuous(durationR);
  } else {
    ExecDiscrete(durationR);
  }
}
